<?php
declare(strict_types=1);

use PieSec\FirebasePhpJwtPoC\KeyManagement;
use Firebase\JWT\JWT;

require_once dirname(__DIR__) . '/vendor/autoload.php';

header('Content-Type: application/json');

if (isset($_GET['token'])) {
    try {
        $decoded = JWT::decode(
            $_GET['token'],
            KeyManagement::auto()->getJwtVerifyingKeyIdMap(),
            ['HS256']
        );
    } catch (Throwable $ex) {
        echo json_encode([
            'status' => 'FAIL',
            'token' => $_GET['token'],
            'ex' => $ex->getMessage(),
            'trace' => $ex->getTrace(),
        ]);
        exit;
    }
    if (!empty($decoded)) {
        echo json_encode([
            'status' => isset($decoded->pwned)
                ? 'Exploit Successful!'
                : 'Decoded Successful',
            'pieces' => $decoded
        ]);
        exit;
    }
}

echo json_encode(
    [
        'token' => JWT::encode(
            [
                'sub' => 'firebase-php-jwt-proof-of-concept.pie-hosted.com',
                'third-party' => 'foo-bar'
            ],
            KeyManagement::auto()->getJwtSigningKeyForAlg('HS256'),
            'HS256',
            'gandalf0'
        )
    ],
    JSON_PRETTY_PRINT
);
