<?php
declare(strict_types=1);

use PieSec\FirebasePhpJwtPoC\KeyManagement;
use Firebase\JWT\JWT;

require_once dirname(__DIR__) . '/vendor/autoload.php';

header('Content-Type: application/json');

if (isset($_GET['token'])) {
    try {
        $decoded = JWT::decode(
            $_GET['token'],
            KeyManagement::auto()->getJwtVerifyingKeyIdMap(),
            ['RS256']
        );
    } catch (Throwable $ex) {
        echo json_encode([
            'status' => 'FAIL',
            'token' => $_GET['token'],
            'ex' => $ex->getMessage(),
            'trace' => $ex->getTrace(),
        ]);
        exit;
    }
    if (!empty($decoded)) {
        echo json_encode([
            'status' => 'OK',
            'pieces' => $decoded
        ]);
        exit;
    }
}

echo json_encode(
    [
        'token' => JWT::encode(
            [
                'sub' => 'firebase-php-jwt-proof-of-concept.pie-hosted.com',
                'user-id' => 'admin001'
            ],
            KeyManagement::auto()->getJwtSigningKeyForAlg('RS256'),
            'RS256',
            'legolas1'
        ),
        'public-key' => KeyManagement::auto()->getJwtVerifyingKeyIdMap()['legolas1']
    ],
JSON_PRETTY_PRINT
);
