<?php
declare(strict_types=1);
namespace PieSec\FirebasePhpJwtPoC;

class KeyManagement
{
    /** @var string $baseDir */
    protected $baseDir = '';

    /**
     * Force the use of auto()
     */
    protected function __construct()
    {

    }

    public static function auto(): self
    {
        $self = new static();
        $self->baseDir = dirname(__DIR__) . '/config';
        if (!is_readable($self->baseDir . '/hs256.symkey')) {
            file_put_contents($self->baseDir . '/hs256.symkey', $self->generateSymmetricKey());
        }
        if (!is_readable($self->baseDir . '/rs256.secret')) {
            $keys = $self->generateAsymmetricKeyPair();
            file_put_contents($self->baseDir . '/rs256.secret', $keys['sk']);
            file_put_contents($self->baseDir . '/rs256.public', $keys['pk']);
        }
        return $self;
    }

    public function generateSymmetricKey(): string
    {
        return bin2hex(random_bytes(32));
    }

    /**
     * @return array{pk: string, sk: string}
     */
    public function generateAsymmetricKeyPair(): array
    {
        $pkey = openssl_pkey_new([
            "digest_alg" => "sha512",
            "private_key_bits" => 4096,
            "private_key_type" => OPENSSL_KEYTYPE_RSA
        ]);
        $pk = openssl_pkey_get_details($pkey)['key'];
        $sk = '';
        openssl_pkey_export($pkey, $sk);
        return ['pk' => $pk, 'sk' => $sk];
    }

    /**
     * @return array
     */
    public function getJwtVerifyingKeyIdMap(): array
    {
        return [
            'legolas1' => $this->getJwtVerifyingKeyForAlg('RS256'),
            'gandalf0' => $this->getJwtVerifyingKeyForAlg('HS256'),
        ];
    }

    /**
     * @param string $alg
     * @return string
     * @throws \Exception
     */
    public function getJwtSigningKeyForAlg(string $alg): string
    {
        if (empty($this->baseDir)) {
            throw new \Exception('Uninitialized');
        }
        if (in_array($alg, ['HS256', 'HS384', 'HS512'], true)) {
            return file_get_contents($this->baseDir . '/hs256.symkey');
        }
        return file_get_contents($this->baseDir . '/rs256.secret');
    }

    protected function getJwtVerifyingKeyForAlg(string $alg): string
    {
        if (in_array($alg, ['HS256', 'HS384', 'HS512'], true)) {
            return file_get_contents($this->baseDir . '/hs256.symkey');
        }
        return file_get_contents($this->baseDir . '/rs256.public');
    }
}
